/** Function definitions*******************************************/

void initTestPins(){
	LATD |= 0x0000; TRISD &= 0xFFE0;
	LATC |= 0x0000; TRISC &= 0x9FFF;
	putsUART2("Init Test Pin Done\r\n");
	
}	

void initMotorPWM(void){
	//Set Enable, Directions, and Loop Time Pins (A2, A3, A4) as digital outputs
	// Initialize as low
	LATA |= 0x0000; TRISA &= 0xBFF3;
	
	// init OC1 module, on pin D0
	OpenOC1( OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0);
	
	// init Timer2 mode and period (PR2) // set for 20kHz
	//OpenTimer2( T2_ON | T2_PS_1_1 | T2_SOURCE_INT, 0x0F9F);
	OpenTimer2( T2_ON | T2_PS_1_8 | T2_SOURCE_INT, 0x9C3F);
}

void initInterruptController(void){
	// init Timer3 mode and period (PR3) // produces 1ms period
	OpenTimer3( T3_ON | T3_PS_1_4 | T3_SOURCE_INT, 0x4E1F);
	
	mT3SetIntPriority( 6); 	// set Timer3 Interrupt Priority
	mT3ClearIntFlag(); 		// clear interrupt flag
	mT3IntEnable( 1);		// enable timer3 interrupts
}

void initEncoder(void){
	// init Timer4 and Timer5 mode and periods (PR4, PR5)
	OpenTimer4( T4_ON | T4_PS_1_1 | T4_SOURCE_EXT, 0xFFFF); 
	OpenTimer5( T5_ON | T5_PS_1_1 | T5_SOURCE_EXT, 0xFFFF); 

	OpenTimer1(T1_ON | T1_PS_1_256, 0xFFFF);
} 

void initServo(){
	OpenOC2( OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0);
}	

void initGuiArrays(){
	int x;
	for (x = 0; x < ARRAY_SIZE; x++){
		actual_speed_array[x] = 0;
	}
	for (x = 0; x < ARRAY_SIZE; x++){
		set_speed_array[x] = 0;
	}
}

void setAnimalParameters(){
	setFrequency(animal_frequency);
	set_speed = abs(animal_speed*sin(time));
	RCservo = middle_servo - animal_amplitude*sin(time+PI_DIVIDE_2+PI*reverse);
}	

void getActualSpeed(){
	if (ReadTimer4() > 0){
		actual_speed = ReadTimer4();
		WriteTimer4(0);	
	}
	else{
		actual_speed = ReadTimer5();
		WriteTimer5(0);	
	}			
 }

void calculateError(){
	current_error = set_speed - actual_speed;
	difference_error = previous_error - current_error;
	previous_error = current_error;
}	

void setPWM(){
	pwm_duty_cycle = current_error*KP + difference_error*KD + pwm_duty_cycle + OFFSET; // PD Controller
	
	if (pwm_duty_cycle > MAX_RESOLUTION){
		pwm_duty_cycle = MAX_RESOLUTION;
	}
	else if (pwm_duty_cycle < 0){
		pwm_duty_cycle = 0;
	}
	SetDCOC1PWM(pwm_duty_cycle);
}

void initUART2(int pbClk){
 	#define config1 	UART_EN | UART_IDLE_CON | UART_RX_TX | UART_DIS_WAKE | UART_DIS_LOOPBACK | UART_DIS_ABAUD | UART_NO_PAR_8BIT | UART_1STOPBIT | UART_IRDA_DIS | UART_DIS_BCLK_CTS_RTS| UART_NORMAL_RX | UART_BRGH_SIXTEEN
	#define config2		UART_TX_PIN_LOW | UART_RX_ENABLE | UART_TX_ENABLE | UART_INT_TX | UART_INT_RX_CHAR | UART_ADR_DETECT_DIS | UART_RX_OVERRUN_CLEAR	
 
 	// Open UART2 with config1 and config2
 	OpenUART2( config1, config2, pbClk/16/DESIRED_BAUDRATE-1);	// calculate actual BAUD generate value.
 		
 	// Configure UART2 RX Interrupt with priority 2
 	ConfigIntUART2(UART_INT_PR2 | UART_RX_INT_EN);
 	putsUART2("Init Done\r\n");
}

void delayms( unsigned t){
   OpenTimer1(T1_ON | T1_PS_1_256, 0xFFFF);
   while (t--)
   {  // t x 1ms loop
       WriteTimer1(0);
       while (ReadTimer1() < SYS_FREQ/256/1000);
  }
  CloseTimer1();
}

void setFrequency(int frequency_prescaler){ 
  	if (time < PI){
   		setDirection(CLOCKWISE);
	}	
	else if (time > PI){
		setDirection(COUNTER_CLOCKWISE);
	}	
  	time = time + 0.01/frequency_prescaler;

	if (time > 2*PI){
		time = 0;
	}
}

void setDirection(int direction){
	if (direction == COUNTER_CLOCKWISE){
		//set momentum wheel direction
		DIRECTION_PIN_1 = 1;
		DIRECTION_PIN_2 = 0;
	}		
	else{
		//set momentum wheel direction
		DIRECTION_PIN_1 = 0;
		DIRECTION_PIN_2 = 1;
	}
}

void fillArrays(){
	int x;
	
	for (x=0; x<ARRAY_SIZE-1; x++){
		actual_speed_array[x] = actual_speed_array[x+1];
	}	
	actual_speed_array[ARRAY_SIZE-1] = actual_speed;
	
	for (x=0; x<ARRAY_SIZE-1; x++){
		set_speed_array[x] = set_speed_array[x+1];
	}	
	set_speed_array[ARRAY_SIZE-1] = set_speed;
}
