/*******************************************************
NORTHWESTERN UNIVERSITY
ME333 Winter 2010 Project
Fluffbot
Group 25: Tyler Johnson, Noah Pentelovitch, Ren Chung Yu

Updated 3/10/2010
********************************************************/

/** Includes ***************************************************/
#include "HardwareProfile.h"
#include "constants.h"
#include "function_declarations.h"
#include "variables.h"
#include "function_definitions.h"

/** Main Function **********************************************/

int main(void)
{
	int	pbClk = SYSTEMConfigPerformance(SYS_FREQ);	// Configure the proper PB frequency and the number of wait states
		
	INTEnableSystemMultiVectoredInt();	// Allow vector interrupts
	initUART2(pbClk);
	mInitAllLEDs();
	initTestPins();
	initMotorPWM();
	initServo();
	initInterruptController();
	initEncoder();
	initGuiArrays();
	
	while(1){
		//printf("%u\r\n", actual_speed);
	}

	CloseOC1();

} //end main

/** Interrupt Handlers *****************************************/

void __ISR( _TIMER_3_VECTOR, ipl6) T3Interrupt( void){
	
	setAnimalParameters();	//change speed, amplitude, and frequency, depending on animal chosen
	getActualSpeed();		//get momentum wheel speed from encoders
	calculateError();		//calculate the error between target and actual momentum wheel speed
	setPWM();				//change PWM1 to reach desired momentum wheel speed
	SetDCOC2PWM(RCservo);	//set PWM2 to reach desired angle for steering wheel
	mT3ClearIntFlag();		// clear interrupt flag and exit
	
} // T3 Interrupt

// UART 2 interrupt handler
void __ISR(_UART2_VECTOR, ipl7) IntUart2Handler(void)
{
	char data;

	if(mU2RXGetIntFlag()){	// Is this an RX interrupt?
		
	    mU2RXClearIntFlag();	// Clear the RX interrupt Flag
		putcUART2(data);
		data = ReadUART2();		// Echo what we just received.
		
		if (data == 'f'){ 			//in water
			animal_speed = HIGH_SPEED;
			animal_frequency = MEDIUM_FREQUENCY;
			animal_amplitude = LOW_AMPLITUDE;
			putsUART2("I'm swimming!\r\n");
		}
		else if (data == 'e'){ 			//on land
			animal_speed = HIGH_SPEED;
			animal_frequency = HIGH_FREQUENCY;
			animal_amplitude = MEDIUM_AMPLITUDE;
			putsUART2("I'm running!\r\n");
		}
		else if (data == 's'){ 		//on ice
			animal_speed = HIGH_SPEED;
			animal_frequency = MEDIUM_FREQUENCY;
			animal_amplitude = HIGH_AMPLITUDE;	
			putsUART2("I'm sliding!\r\n");	
		}			
 		else if (data == 'g'){ 		//go straight	
	 		reverse = 0;
	 		middle_servo = MIDDLE_SERVO;
	 		putsUART2("Going straight!\r\n");
	 	}	
	 	else if (data == 'l'){ 		//turn left	
	 		middle_servo = MIDDLE_SERVO - TURN_SERVO;
	 		putsUART2("Turning left!\r\n");
	 	}	
	 	else if (data == 'r'){ 		//turn right	
	 		middle_servo = MIDDLE_SERVO + TURN_SERVO;
	 		putsUART2("Turning right!\r\n");
	 	}
	 	else if (data == 'p'){ 		//stop
		 	animal_speed = 0;
		 	animal_frequency = 0;
			animal_amplitude = 0;
	 		middle_servo = MIDDLE_SERVO;
	 		putsUART2("Stopped!\r\n");
	 	}
	 	else if (data == 't'){ 		//reverse
			reverse = 1;
	 		putsUART2("Reverse!\r\n");
	 	}
	}
	else if ( mU2TXGetIntFlag() ){	// We don't care about TX interrupt
		mU2TXClearIntFlag();
	}
}
